<?php declare (strict_types = 1);

namespace EuKit\Base\Parts;

use OcArticle;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Base\Models\Post;
use Infomaker\Everyware\Base\PropertyParser;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Base\ArticleBodyParser;
use Infomaker\Everyware\Base\OcPropertyMapConfig;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use Tightenco\Collect\Support\Collection;

/**
 * Class NewsMLArticle
 * @property string     headline
 * @property array      channels
 * @property Date       pubdate
 *
 * @property Collection articles
 * @property Collection authors
 * @property Collection categories
 * @property Collection places
 * @property Collection stories
 * @property Collection tags
 *
 * @package EuKit\Base
 */
class NewsMLArticle extends GenericPropertyObject
{
    public function __construct(PropertyObject $object)
    {
        $this->fill($object->getProperties());
    }

    /**
     * Contains an instance of the parser for the article
     *
     * @var ArticleBodyParser
     */
    protected $parser;

    public function getParsedBody(): ArticleBodyParser
    {
        return $this->getBodyParser();
    }

    public function getBodyParser(): ArticleBodyParser
    {
        if ($this->parser === null) {
            $this->parser = new ArticleBodyParser($this);
        }

        return $this->parser;
    }


    private function getNewsMLObject($newsml)
    {
        if (empty($newsml)) {
            return [];
        }

        $transformer = NewsMLTransformerManager::createObjectTransformer();

        $result = $transformer->transform($newsml);

        return count($result) > 0 ? array_shift($result) : [];
    }

    public function getNewsMLTeaser()
    {
        return $this->getNewsMLObject($this->teaser_raw);
    }

    public function getNewsMLSocialTeaser()
    {
        if (!empty($this->getSingleValue("social_teaser_raw"))) {
            return $this->getNewsMLObject($this->social_teaser_raw);
        } else {
            // Default values to x-im/teaser
            return $this->getNewsMLTeaser();
        }
    }

    /**
     * @return bool
     */
    public function hasRelatedArticles(): bool
    {
        return $this->getRelatedArticles()->isNotEmpty();
    }

    /**
     * Retrieve a list of Place concepts without the hierarchical relations
     *
     * @since 1.0.0
     * @return Collection
     */
    public function getFilteredPlaces(): Collection
    {
        $places = $this->getPlaces();
        $placeRelationUuids = \array_column( $places->toArray(), 'parent_uuid', 'uuid' );

        $placesToKeep = $placeRelationUuids;

        foreach ( $placeRelationUuids as $uuid => $parentUuid ) {
            // Remove my parent
            unset( $placesToKeep[$parentUuid] );
        }

        return $places->whereIn( 'uuid', \array_keys( $placesToKeep ) );
    }

    /**
     * @return Collection
     */
    public function getArticleTags(): Collection
    {
        return $this->getFilteredPlaces()
            ->merge( $this->getTags() )
            ->unique( 'name' )
            ->sortBy( 'name' );
    }

    /**
     * @return Collection|null
     */
    public function getChannels(): ?Collection
    {
        return Collection::make( $this->channels );
    }

    /**
     * Retrieve all Place-concepts
     *
     * @return Collection
     */
    public function getPlaces(): Collection
    {
        return Collection::make( $this->places )->unique( 'name' );
    }

    /**
     * @return Collection
     */
    public function getAuthors(): Collection
    {
        return Collection::make( $this->authors )->unique( 'name' );
    }

    /**
     * @return Collection
     */
    public function getStories(): Collection
    {
        return Collection::make( $this->stories )->unique( 'name' );
    }

    /**
     * @return Collection
     */
    public function getSections(): Collection
    {
        return Collection::make( $this->sections )->unique( 'name' );
    }

    /**
     * @return Collection
     */
    public function getCategories(): Collection
    {
        return Collection::make( $this->categories )
            ->unique( 'name' );
    }

    /**
     * @return Collection
     */
    public function getTags(): Collection
    {
        return Collection::make( $this->tags )
            ->unique( 'name' );
    }

    /**
     * Retrieve related article by uuid
     *
     * @param string $uuid
     *
     * @return static|null
     */
    public function getRelatedArticle(string $uuid)
    {
        $articles = $this->getRelatedArticles()->whereIn('uuid', $uuid);

        return $articles->isNotEmpty() ? $articles->first() : null;
    }

    /**
     * @return Collection
     */
    public function getRelatedArticles(): Collection
    {
        return Collection::make( $this->articles );
    }

    public function getPubDate(): Date
    {
        return Date::parse($this->pubdate);
    }

    public static function createFromPost(Post $post, $propertyMap = 'Article')
    {
        $uuid = $post->getMeta('oc_uuid');

        $provider = OpenContentProvider::setup();
        $provider->setPropertyMap($propertyMap);
        $article = $provider->getObject($uuid);

        return $article ? new static($article) : null;
    }

    public static function createfromOcObject(OcArticle $article, $propertyMap = 'Article'): self
    {
        $conf = new OcPropertyMapConfig();
        $propertyParser = new PropertyParser($conf);

        return new static($propertyParser->fromOcObject($propertyMap, $article));
    }
}
